Function value_defined
fn value_defined(tester_name: &str, value: Option<&Value>) -> Result<()>
Called to check if the Value is defined and return an Err if not
value_definedfn value_defined(tester_name: &str, value: Option<&Value>) -> Result<()>
Called to check if the Value is defined and return an Err if not